////////////////////////////////////////////////////////////////
//                                                            //
//                   File Renaming Algorithm                  //
//                 by LD Blake and Erol Ojalvo                //
//                          June 2016                         //
//                                                            //
////////////////////////////////////////////////////////////////

// be sure to include these files
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <locale.h>
#include <shlwapi.h>
#include <lmcons.h>

// Old is the input filename, without path or drive information
// New is the output filename after processing by macros.
// WCard is the macro string used to rename files.

// returns 1 for success, 0 on errors

BOOL RenameFile(PCHAR Old, PCHAR WCard, PCHAR New)
  {
    PCHAR ext  = PathFindExtension(Old); 
    PCHAR nptr = New;  // newname pointer
    PCHAR optr = Old;  // oldname pointer

    // copy name if no wildcard
    if(strlen(WCard) < 2) 
      {
        strcpy(New, Old);
        return 1;
      }

    while(*WCard)
      {
        switch(*WCard)
          {
            case '*' :                            // transformations
              {
                PCHAR sptr = optr; // source 
                PCHAR dptr = optr; // dest

                switch(toupper(*(++WCard)))
                  {
                    case 'A':     // remove all whitespace
                      while(sptr < ext)
                        {
                          if(! isspace(*sptr))
                            *(dptr++) = *(sptr);
                          ++sptr;
                        }
                      break;   

                    case 'B' :    // burst PrettyPath
                      while(dptr < ext)
                        {
                          if(dptr > optr && isupper(*dptr) && (!isspace(*(dptr - 1))))
                            {
                              memmove(dptr + 1, dptr, strlen(dptr) + 1);
                              *dptr = ' ';
                              ++ext;
                              dptr += 2;
                            }
                          else
                            ++dptr;
                        }
                      break;

                    case 'C' :    // convert to ALL CAPS
                      while(dptr < ext)
                        *dptr = (char) toupper(*(dptr++));
                      break;

                    case 'D' :    // spaces to dashes
                      while(dptr < ext)
                        {
                          if(isspace(*dptr))
                            *dptr = '-';
                          ++dptr;
                        }
                      break;

                    case 'L' :    // convert to lower case
                      while(dptr < ext)
                        *dptr = (char) tolower(*(dptr++));
                      break;

                    case 'M' :    // punctuation marks to spaces
                      while(dptr < ext)
                        {
                          if(ispunct(*dptr))
                            *(dptr) = ' ';
                          ++dptr;
                        }  
                      break;   

                    case 'P' :    // convert to PrettyPath
                      while(dptr < ext) 
                        {
                          if((isalpha(*dptr)) && (!isalpha(*(dptr - 1)) || dptr == optr))
                            *dptr = (char) toupper(*dptr);
                          else
                            *dptr = (char) tolower(*dptr);
                          ++dptr;
                        }
                      // pass 2: remove spaces
                      dptr = optr;
                      while(sptr < ext)
                        {
                          if(! isspace(*sptr))
                            *(dptr++) = *(sptr);
                          ++sptr;
                        }
                      break;   

                    case 'S' :    // convert to sentence case
                      while(dptr < ext && (! isalpha(*dptr)))
                        dptr++;
                      *dptr = toupper(*(dptr++));
                      while(dptr < ext)
                        *dptr = tolower(*(dptr++));
                      break;

                    case 'T' :    // convert to title case
                      while(dptr < ext)
                        {
                          if((isalpha(*dptr)) && (!isalpha(*(dptr - 1)) || dptr == optr))
                            *dptr = (char) toupper(*dptr);
                          else
                            *dptr = (char) tolower(*dptr);
                          ++dptr;
                        }
                      break;

                    case 'U' :    // spaces to underscores
                      while(dptr < ext)
                        {
                          if(isspace(*dptr))
                            *dptr = '_';
                          ++dptr;
                        }
                      break;

                    case 'W':     // remove excess whitespace
                      while(sptr < ext)
                        {
                          if(isspace(*sptr))
                            {
                              if(dptr > optr && *(sptr - 1) != ' ')
                                *(dptr++) = ' ';
                            }
                          else
                            *(dptr++) = *sptr;
                          sptr++;
                        }
                      // pass 2: trailing whitespace  
                      while(dptr > optr && isspace(*(dptr - 1)))
                        --dptr;                                
                      break;                    

                    default :
                      return 0;
                  }

                //dptr points to end of name, copy extension
                if(dptr < ext)
                  {
                    sptr = ext;
                    ext = dptr;
                    while(*sptr)
                      *(dptr++) = *(sptr++);
                    *dptr = 0;
                  }
                break;
              }                  

            case '?' :                            // editing macros
              {
                switch(toupper(*(++WCard)))
                  {
                    case 'A' :    // insert account name
                      {
                        DWORD nsz = LM20_UNLEN + 1;
                        GetUserName(nptr, &nsz); 
                        if(nsz > 0)
                          nptr += nsz - 1;
                      }
                      break;

                   case 'C' :     // copy to character 
                      ++WCard;
                      do
                        *(nptr++) = *(optr++);
                      while(*optr != *WCard && optr < ext);
                      if(optr < ext)
                        *(nptr++) = *(optr++);
                      break;

                    case 'D' :    // insert date
                      {
                        SYSTEMTIME st = {0};

                        GetLocalTime(&st);
                        nptr += sprintf(nptr, "%4d-%02d-%02d", st.wYear, st.wMonth, st.wDay);
                      }
                      break;
      
                    case 'E' :    // copy extension
                      {
                        PCHAR eptr = ext;
                        while(*eptr)
                          *(nptr++) = *(eptr++);
                      }
                      break;

                    case 'F' :    // copy filename
                      while(*optr && optr != ext)
                        *(nptr++) = *(optr++);
                      break;

                    case 'H' :    // insert host name
                      {
                        DWORD nsz = LM20_SNLEN + 1;
                        GetComputerName(nptr, &nsz); 
                        if(nsz > 0)
                          nptr += nsz;
                      }
                      break;

                    case 'I' :    // copy next character
                      if(optr < ext)
                        *(nptr++) = *(optr++);
                      break;

                    case 'J' :    // jump over word
                      while(optr <ext && isspace(*optr))
                        ++optr;
                      while(optr < ext && !isspace(*optr))
                        ++optr;
                      while(optr <ext && isspace(*optr))
                        ++optr;
                      break;

                    case 'L' :    // copy as lower case
                      if(optr < ext)
                        *(nptr++) = (char) tolower(*(optr++));
                      break;

                    case 'N' :    // copy filename and extension
                      while(*optr)
                        *(nptr++) = *(optr++);
                      break;

                    case 'O' :    // omit next character
                      if(optr < ext)
                        ++optr;
                      break;

                    case 'S' :              // skip to character
                      ++WCard;
                      do
                        ++optr;
                      while(*optr != *WCard && optr < ext);
                      ++optr; 
                      break;

                    case 'T' :    // insert time
                      {
                        SYSTEMTIME st = {0};

                        GetLocalTime(&st);
                        nptr += sprintf(nptr, "%02d-%02d-%02d", st.wHour, st.wMinute, st.wSecond);
                      }
                      break;
  
                    case 'U' :    // copy as upper case
                      if(optr < ext)
                        *(nptr++) = (char) toupper(*(optr++));
                      break;

                    case 'W' :    // copy next word
                      while(optr <ext && isspace(*optr))
                        ++optr;
                      while(optr < ext && !isspace(*optr))
                        *(nptr++) = *(optr++);
                      break;
                        
                    default :
                      return 0;
                  }
                break;
              }

            default:
              *(nptr++) = *(WCard);
          }

        *nptr = 0; // keep terminated
        ++WCard; 
      }
    return 1;
  }

